/******************************************************************/
/*                                                                */
/*                    Reading Citibase Data                       */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: loaddat2.g
** created:  08/23/99
*/

/* Time series, extracted 07/23/98
** GDPQ_CL = log per capita GDP
** GCIQ_CL = log per capita consumption + investment
** GCNSQ_CL = log per capita consumption
** LPMHU_CL = hours
** BLSHOURS_CL
** LHours_CL
** Quarterly Data from 1954:2 to 1997:4
*/

load data_q[175,6]="c:/projects/active/persist/data/data1.asc";

gdpq_cl   = data_q[.,1];
gciq_cl   = data_q[.,2];
gcnsq_cl  = data_q[.,3];
lpmhu_cl  = data_q[.,4];
blsh_cl   = data_q[.,5];
lhours_cl = data_q[.,6];

nobs = rows(data_q);

/* Difference Data and eliminate the 1954:2 observation
** Remaining Sample: 1954:III to 1997:IV
** 174 Observations
*/
gdpq_cld = gdpq_cl[2:nobs] - gdpq_cl[1:nobs-1];
gciq_cld = gciq_cl[2:nobs] - gciq_cl[1:nobs-1];
gcnsq_cld= gcnsq_cl[2:nobs] - gcnsq_cl[1:nobs-1];
lpmhu_cl  = lpmhu_cl[2:nobs];
blsh_cl   = blsh_cl[2:nobs];
lhours_cl = lhours_cl[2:nobs];

series = gdpq_cld ~ blsh_cl;
nobs   = rows(series);

trd   = 0;               /* trd = 0: no time trend, trd = 1 with time trend */
nvar  = 2;               /* number of dependent vars        */
nlags = 4;               /* number of lags                  */
k     = 1 + trd + nvar*nlags;  /* number of coefficients per equation  */
T0    = 22;              /* number of obs to initialize prior and lags */


ti       = seqa(1954.5,0.25,nobs);
YY       = series[nlags+1:nobs,.];
XX       = ones(nobs-nlags, k);
if trd == 1;
   XX[.,2]  = seqa(-T0+nlags,1,nobs-nlags);
endif;
ti       = ti[nlags+1:nobs,.];

i = 1;
do until i > nlags;
   XX[.,2+trd+(i-1)*2:1+trd+i*2] = series[nlags+1-i:nobs-i,.];
   i = i+1;
endo;

/* Adjust sample size for the initialization of lags
*/
nobs = nobs-nlags;
T0   = T0-nlags;
 
/* Specify Minnesota Prior: Hyperparameter grid, intercept tightness
**
*/
n_p1    = 20;
p1_l    = 0.001;
p1_u    = 10;
p1_grid = exp( seqa(ln(p1_l), ln(p1_u/p1_l)/(n_p1-1), n_p1) );

intcpt_v0 = 100;

{C_bar, CC_bar, VCm_inv} = minprior(1,YY[1:T0,.],XX[1:T0,.]);


/******************************************************************/
/*                  P R O C E D U R E S                           */
/******************************************************************/

proc (3) = minprior(lambda, YY, XX);
           local C_bar, CC_bar, VC_inv, y, i,
           c_hat, CC_hat, SIG, SIG_hat, VC_diag, k, nvar, nobs;

/* Prior Mean
*/
k = cols(XX);
nvar   = cols(YY);
CC_bar = zeros(nvar,k);
C_bar  = vec(CC_bar');
nobs   = rows(YY);

/* Prior Variance
*/
y  = vec(YY);
XX = XX[.,1:3+trd];  /* Use only VAR(1) here */
c_hat   = (eye(nvar) .*. (inv(XX'*XX)*XX'))*y;
CC_hat  = reshape(c_hat,nvar,3+trd);
SIG_hat = diag(1/nobs *(YY'-CC_hat*XX')*(YY'-CC_hat*XX')');
SIG     = (ones(nvar,1)./SIG_hat)*SIG_hat';

VC_inv  = ones(nvar,1+trd)/intcpt_v0;
i = 1;
do until i > nlags;
   VC_inv = VC_inv~SIG.*(i/lambda)^2;
   i = i+1;
endo;

VC_inv = eye(nvar*k).*vec(VC_inv');
retp(C_bar,CC_bar, VC_inv);
endp;

