/******************************************************************/
/*                                                                */
/*                    Reading Citibase Data                       */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: loaddata.g
** created:  08/23/99
*/

/* Time series, extracted 07/23/98
** GDPQ_CL = log per capita GDP
** GCIQ_CL = log per capita consumption + investment
** GCNSQ_CL = log per capita consumption
** LPMHU_CL = hours
** BLSHOURS_CL
** LHours_CL
** Quarterly Data from 1954:2 to 1997:4
*/

load data_q[175,6]="c:/projects/active/persist/data/data1.asc";

gdpq_cl   = data_q[.,1];
gciq_cl   = data_q[.,2];
gcnsq_cl  = data_q[.,3];
lpmhu_cl  = data_q[.,4];
blsh_cl   = data_q[.,5];
lhours_cl = data_q[.,6];

nobs = rows(data_q);

/* Difference Data and eliminate the 1954:2 observation
** Remaining Sample: 1954:III to 1997:IV
** 174 Observations
*/
gdpq_cld = gdpq_cl[2:nobs] - gdpq_cl[1:nobs-1];
gciq_cld = gciq_cl[2:nobs] - gciq_cl[1:nobs-1];
gcnsq_cld= gcnsq_cl[2:nobs] - gcnsq_cl[1:nobs-1];
lpmhu_cl  = lpmhu_cl[2:nobs];
blsh_cl   = blsh_cl[2:nobs];
lhours_cl = lhours_cl[2:nobs];

T0     = 22;     /* number of obs to initialize prior */
Tstart = T0+1;   /* Tstart is first observation, prev obs initialize lags */

series = gdpq_cld ~ blsh_cl;
YY     = series;
nobs   = rows(YY);
trend  = seqa(-Tstart+1,1,nobs); 
 
/* Load Prior for mu and phi (quarterly) from panel data analysis (mhrun = "1")
*/
ppath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";
psummary = ppath $+ "mu_phiq1" $+"s";

open fhpsummary = ^psummary for read;
muphi_m0 = readr(fhpsummary,1)';
muphi_s0 = readr(fhpsummary,1)';
muphi_v0 = readr(fhpsummary,2);
closeall fhpsummary;

/* Set path and import the information for the prior density
*/

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\para";
lmodel = "m" $+ ftos(mspec,"%*.*lf",1,0);
lprior = ftos(mprior,"%*.*lf",1,0);

priorfile = lmodel $+ lprior $+ "pri.out";
load path=^lpath prior[npara+1,3] = ^priorfile;
/* Note: there are npara+1 rows in the prior configuration file
**       the last row determines the scale of the prior for MU and PHI
**       The entries in rows 7 and 8 have no effect on the prior of
**       MU and PHI, because the values in the third column are 0.
**       The joint prior for MU and PHI was loaded seperately (see above).
*/


pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
muphi_scale = prior[npara+1,1];
muphi_v0    = muphi_v0*muphi_scale^2;

