/******************************************************************/
/*                                                                */
/*         Compare IRFs and Corr for different parameters         */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: efftpred.g
** created:  10/13/99
** 
*/

new;
library lbdlib, user, pgraph;
cls;

mspec = 2;
nirf  = 40;                 /* number of impulse responses */
ncov  = 4;

irf_ti    = seqa(1,1,nirf);
cor_ti    = seqa(1,1,ncov);


irf_label = "Output / Trans Shock" $|
            "Hours / Trans Shock"  $|
            "Output / Perm Shock"  $|
            "Hours / Perm Shock";

irf_yticinfo = ( -0.2 ~ 1   ~ 0.2 ~ 2 )|
               ( -0.2 ~ 1   ~ 0.2 ~ 2 )|
               ( -0.2 ~ 1.6 ~ 0.2 ~ 2 )|
               ( -0.2 ~ 1   ~ 0.2 ~ 2 );

cor_label = "Corr( dy, dy(-j) )" $|
             "Corr( h, h(-j) )";

cor_yticinfo = ( -0.1 ~ 0.4 ~ 0.1 ~ 2 )|
                ( 0.5 ~ 1   ~ 0.1 ~ 2 );

/*******************************************************************
** Load VAR Responses
*/
mhrun = "1";

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lsummary       = lpath $+ "m01c12" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic12         = readr(fhsummary,1)';
closeall fhsummary;

lsummary       = lpath $+ "m01c22" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic22         = readr(fhsummary,1)';
closeall fhsummary;

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lsummary       = lpath $+ "m01c11" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic11         = readr(fhsummary,1)';
closeall fhsummary;

lsummary       = lpath $+ "m01c21" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic21         = readr(fhsummary,1)';
closeall fhsummary;

lsummary       = lpath $+ "m01cory" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0cory         = readr(fhsummary,1)';
closeall fhsummary;

lsummary       = lpath $+ "m01corh" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0corh         = readr(fhsummary,1)';
closeall fhsummary;

m0cory = m0cory[1:4];
m0corh = m0corh[1:4];

/*******************************************************************
** Load Posterior Mean Parameter estimates for LBD Model and use as
** starting values
** alp, bet, gam, del, nu, rho, mu, phi, chi, psi, siga, sigb
**  1    2    3    4    5   6    7   8    9    10   11    12
*/
mhrun = "1";
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\para";
load path = ^lpath para_names;

osummary = lpath $+ "\\m31mode";
open fhsummary = ^osummary for read;
m3para = readr(fhsummary,12);
closeall fhsummary;

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
osummary       = lpath $+ "m21par" $+ mhrun $+ "s";
open fhsummary = ^osummary for read;
m2para         = readr(fhsummary,1)';
npara          = rows(m2para);
closeall fhsummary;

/********************************************************************
** Print the Estimation Results
*/
cls;

"Parameter   | M2  |  M3  ";
outmat = para_names'~m2para~m3para;
let mask[1,3] = 0 1 1;
let fmt[3,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);


/********************************************************************
** Compute Impulse response functions and correlations
** 12 22 21 22
*/

{m2irfda, valid} = dsgeirf( m2para,2,nirf, (100*m2para[11]|0) );
{m2irfdb, valid} = dsgeirf( m2para,2,nirf, (0|-100*m2para[12]) );

{m3irfda, valid} = dsgeirf( m3para,3,nirf, (100*m3para[11]|0) );
{m3irfdb, valid} = dsgeirf( m3para,3,nirf, (0|-100*m3para[12]) );

  
{m2cory, m2corh, valid} = dsgemom(m2para,2);
{m3cory, m3corh, valid} = dsgemom(m3para,2);

m2cory = m2cory[1:4]';
m3cory = m3cory[1:4]';
m2corh = m2corh[1:4]';
m3corh = m3corh[1:4]';


/********************************************************************
** Plot impulse response functions and autocorrelation functions
*/
   graphset;
   begwind;
   margin(0,0,0.2,0.2);
   window(3,2,0);
   fonts("microb");
   _ptitlht = 0.4;
   _paxht = 0.4;
   _pnumht = 0.3;
   _pcolor =2 ;
   _pltype = {1 6 2 3};
   _protate = 1;
   _plctrl = 0;
   _plwidth = 3.5;
   _psymsiz = 5;
   _plegstr = "LBD Prior 2\000Effort Model\000VAR (Post. Mean)";
   _plegctl = {2 4 2.7 5.2};

      i = 1; 
      ylabel(irf_label[i]);  
      ytics(irf_yticinfo[i,1], irf_yticinfo[i,2], irf_yticinfo[i,3], irf_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 5.2};
      xy(irf_ti,m2irfdb[.,1]~m3irfdb[.,1]~m0ic12);
      nextwind;

      i = 2; 
      ylabel(irf_label[i]);  
      ytics(irf_yticinfo[i,1], irf_yticinfo[i,2], irf_yticinfo[i,3], irf_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 5.2};
      xy(irf_ti,m2irfdb[.,3]~m3irfdb[.,3]~m0ic22);
      nextwind;

      i = 3; 
      ylabel(irf_label[i]);  
      ytics(irf_yticinfo[i,1], irf_yticinfo[i,2], irf_yticinfo[i,3], irf_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 2.2};
      xy(irf_ti,m2irfda[.,1]~m3irfda[.,1]~m0ic11);
      nextwind;

      i = 4; 
      ylabel(irf_label[i]);  
      ytics(irf_yticinfo[i,1], irf_yticinfo[i,2], irf_yticinfo[i,3], irf_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 5.2};
      xy(irf_ti,m2irfda[.,3]~m3irfda[.,3]~m0ic21);
      nextwind;

      i = 1;
      ylabel(cor_label[1]);  
      ytics(cor_yticinfo[i,1], cor_yticinfo[i,2], cor_yticinfo[i,3], cor_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 5.2};
      xy(cor_ti,m2cory~m3cory~m0cory);
      nextwind;

      i = 2;
      ylabel(cor_label[1]);  
      ytics(cor_yticinfo[i,1], cor_yticinfo[i,2], cor_yticinfo[i,3], cor_yticinfo[i,4]);   
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      _plegctl = {2 4 2.7 2.2};
      xy(cor_ti,m2corh~m3corh~m0corh);
      endwind;



/*************************************************************************/
/*                           PROCEDURES                                  */
/*************************************************************************/

proc (2) = dsgeirf( para,mspec,nsim,sig );
/* This procedure generates impulse response functions
*/
local valid, TTT, RRR, retcode, ss, irf, t, nvar, zt;

valid = 1;
/* Call procedure to solve the model
*/
{TTT, RRR, retcode} = lbdsolv(para,mspec);
if retcode > 0;
   valid = 0;
   irf = 0;
   goto eoirf;
endif;

nvar = cols(TTT);

ss = zeros(nsim+1,nvar);
zt = zeros(nsim+1,1);
irf = zeros(nsim+1,3);

ss[2,.] = ( TTT*ss[1,.]' + RRR*sig )';
zt[2,.] = zt[1,.] + (1~0)*sig;
irf[2,.] = (ss[2,1]+zt[2,.])~(ss[2,1]+zt[2,.]-ss[2,2])~ss[2,3];

t = 3;
do until t > nsim+1;
   ss[t,.] = ( TTT*ss[t-1,.]' )';
   zt[t,.] = zt[t-1,.];
   irf[t,.] = (ss[t,1]+zt[t,.]) ~ ( ss[t,1]-ss[t,2]) ~ ss[t,3];
   t = t+1;
endo;

irf = irf[2:nsim+1,.];
eoirf:
retp(irf, valid);
endp;

/*************************************************************************/


proc (3) = dsgemom(para,mspec);
local TTT, RRR, valid, nstate, ZZ, HH, QQ, VV, GAMM0, CORR0, GA0,
      Gl, Gstar, TTl, l, cory, corh, siga, sigb, retcode, npara;
 
     npara = rows(para);
     siga = para[npara-1,1];
     sigb = para[npara];
     valid = 1;

     /* Compute autocovariances
     */
     {TTT, RRR, retcode} = lbdsolv(para, mspec);
     if retcode > 0;
        valid = 0;
        cory = zeros(1,ncov);
        corh = zeros(1,ncov);
        goto invalid;
     endif;
     nstate = rows(TTT);
     /* State vector is of the form dy(t), dy(t-1), dlm(t), ...
     */
     ZZ = zeros(2,nstate);
     ZZ[1,1] = 1;
     ZZ[1,2] = -1;
     ZZ[2,3] = 1;
   
     HH = ( siga^2 ~ 0 )|( 0 ~ 0     );
     QQ = ( siga^2 ~ 0 )|( 0 ~ sigb^2); 
     VV = ( siga^2 ~ 0 )|( 0 ~ 0     );

     /* Initialize Autocovariances
     */
     GAMM0 = zeros(4,ncov+1);
     CORR0 = zeros(4,ncov+1);

     GA0  = inv(eye(nstate^2) - (TTT.*.TTT))*vec(RRR*QQ*RRR');
     GA0  = reshape(GA0,nstate,nstate);

     Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
     Gstar= ( Gl[1,1] ~ sqrt(Gl[1,1]*Gl[2,2]) )|
          ( sqrt(Gl[1,1]*Gl[2,2]) ~ Gl[2,2] );

     GAMM0[.,1] = vecr(Gl);
     CORR0[.,1] = vecr(Gl./Gstar);
 
     TTl = TTT;
     l=1;
     do until l > ncov;
      Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);

      GAMM0[.,l+1] = vecr(Gl);           
      CORR0[.,l+1] = vecr(Gl./Gstar);
      TTl = TTl*TTT;
      l = l+1; 
     endo;

     cory = CORR0[1,2:ncov+1];
     corh = CORR0[4,2:ncov+1];
     invalid:

retp( real(cory), real(corh) ,valid);
endp;
