/* filename:    lbdfilt.g
** description: The program filters out the technology processes
** created:     21/10/99
*/

new;
closeall;
library user, pgraph, lbdlib;
cls;

/******************************************************** 
**      Specify the DSGE Model
**      Models: 1 = RBC
**              2 = LBD
**              3 = LBD + Effort
*/ 
mspec  = 2;
mprior = 2;
npara  = 12;
mhrun  = "1";

/* Define some Constants
*/
nburn  = 10001;       /* burn-in */
nsim   = 90;          /* block size */
skip   = 1;

/******************************************************** 
** Import data on output growth and inflation: series (nobs,2)
** observations from 1954:III to 1997:IV: 
**
** YY is composed of gdpq_cld and blsh_cl
*/
#include c:\projects\active\persist\Gauss\prog_t03\loaddata.g
loadm path = ^lpath para_names;

/******************************************************** 
** Open files with Parameter Draws
*/
npara = 12;
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
opath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";

lpara = lpath $+ lmodel $+ lprior $+ "par" $+ mhrun;
open fhpara = ^lpara for read;

/* Initialize Output files
*/
oat = opath $+ lmodel $+ lprior $+ "at" $+ mhrun;
oxt = opath $+ lmodel $+ lprior $+ "xt" $+ mhrun;

create fhat = ^oat with AT, nobs-T0, 8;
create fhxt = ^oxt with XT, nobs-T0, 8;

/* Initialization
*/ 

eofloop = 0;
loopct  = 1;
pararow = seekr(fhpara,nburn);
totaldrawsused = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nsim );

   nread = rows(parasim);
   
   if skip == 0;   
      Atsim = zeros(nread,nobs-T0);
      btsim = zeros(nread,nobs-T0);
      xtsim = zeros(nread,nobs-T0);
   else;
      Atsim = zeros(1,nobs-T0);
      btsim = zeros(1,nobs-T0);
      xtsim = zeros(1,nobs-T0);
   endif;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, conduct the filtering
      */   

     statesim = lbdfilt( parasim[j,.]', mspec, YY);
   
     Xtsim[j,.] = parasim[j,1]*statesim[T0+1:nobs,1]';
     Atsim[j,.] = parasim[j,1]*statesim[T0+1:nobs,2]'; /*+ ( parasim[j,3]*seqa(1,1,nobs-T0) )'; */
     totaldrawsused = totaldrawsused+1;

     locate 1,1;
     "Block      " loopct;
     "Iteration  " j "of" nsim;
     "Draws used " totaldrawsused;

     if skip == 1;
        j = rows(parasim)+2;
     else;
        j = j+1;
     endif;
  endo;

  /* Write Output
  */
  wr = writer(fhat,Atsim);
  wr = writer(fhxt,Xtsim);

  loopct = loopct + 1;
  eofloop = eof(fhpara);

endo;

closeall fhat, fhxt, fhpara;


