/* filename:    lbdhess.g
** description: The program computes the hessian at the posterior mode 
** created:     05/05/00
*/

new;
closeall;
library user, pgraph, lbdlib;
format /mb1 /ros 16,8;
cls;

/******************************************************** 
**      Estimate the DSGE Model
**      Models: 1 = RBC
**              2 = LBD
**              3 = LBD + Effort
*/ 
mspec  = 3;
mprior = 2;
npara  = 12;

/******************************************************** 
** Import data on output growth and inflation: series (nobs,2)
** observations from 1954:III to 1997:IV: 
**
** YY is composed of gdpq_cld and blsh_cl
*/
#include c:\projects\active\persist\Gauss\prog_t03\loaddata.g
loadm path = ^lpath para_names;

/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ "mode";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

"";
"Prior*Likelihood at Mode";
fcn(para);
"Press Key to Continue";
k = keyw;
cls;

/*
goto evalhess;
*/

/* Compute Hessian, element by element, fine tune with dxscale
*/
comphess:

ndx = 6;
dx =  exp(-seqa(6,2,ndx));
hessian = zeros( npara, npara );
gradx = zeros(ndx,1);
grady = zeros(ndx,1);
gradxy = zeros(ndx,1);
hessdiag = zeros(ndx,1);
dxscale = ones(npara,1);


/* Compute Diagonal elements first
*/
seli = 1;
do until seli > npara;
     locate 1,1;
     "Hessian Element    (" seli seli ")";
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[seli] = parady[seli] - dx[i]*dxscale[seli];
      paradxdy = paradx;
      paradxdy[seli] = paradxdy[seli] - dx[i]*dxscale[seli];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[seli]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[seli])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[seli]);
      i = i+1;
     endo;
     "Values";
     -hessdiag';
     hessian[seli,seli] = -0.5*(hessdiag[3]+hessdiag[4]);
     locate 6,1;
     "Value Used:" hessian[seli,seli];
   seli = seli+1;
endo;

/* Now compute off-diagonal elements
** Make sure that correlations are between -1 and 1
** errorij contains the index of elements that are invalid
*/
errorij = 0~0~0;

seli = 1;
do until seli == npara;
   selj = seli+1;
   do until selj > npara;
     locate 1,1;
     "Hessian Element    (" seli selj ")";
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[selj] = parady[selj] - dx[i]*dxscale[selj];
      paradxdy = paradx;
      paradxdy[selj] = paradxdy[selj] - dx[i]*dxscale[selj];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[selj]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[selj])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[selj]);
      i = i+1;
     endo;
     "Values";
     -hessdiag';

     hessian[seli,selj] = -0.5*(hessdiag[3]+hessdiag[4]);
     
     if ( hessian[seli,seli] == 0 ) or (hessian[selj,selj] == 0);
        corrij = 0;
     else;
        corrij = hessian[seli,selj]/sqrt(hessian[seli,seli]*hessian[selj,selj]);
     endif;

     if (corrij < -1) OR (corrij > 1);
        hessian[seli,selj]=0;
        errorij = errorij |(seli~selj~corrij);
     endif;   
     hessian[selj,seli] = hessian[seli,selj];

     locate 6,1;
     "Value Used: " hessian[seli,selj];
     "Correlation:" corrij;
     "Number of Errors:" rows(errorij)-1;
     selj=selj+1;
   endo;
   seli = seli+1;
endo;

cls;
"Errors"  errorij;

/* Initialize Output files
*/
ohess = lpath $+ "\\" $+ lmodel $+ lprior $+ "hess";
create fhhess=^ohess with hess, npara, 8;
wr    = writer(fhhess,hessian[1:npara,1:npara]);
closeall fhhess;
   


/*******************************************************************************
*/

/* Load Hessian, compute penalty
*/
evalhess:

lhess = lpath $+ "\\" $+ lmodel $+ lprior $+ "hess";
open fhhess = ^lhess for read;
HHm   = readr( fhhess,npara );
closeall fhhess;

/* Hessian is of reduced rank. Keep zero rows and columns and do SVD
*/
if mspec == 1;
   rankHHm = 9;
elseif mspec == 2;
   rankHHm = 11;
else;
   rankHHm = 12;
endif;

/* Create Inverse by Singular Value Decomposition
*/
{u , s, v} = svd1(HHM);
invHHMdet = 1;

i = 1;
do until i > npara;
   if i > rankHHM;
      s[i,i] = 0;
   else;
      s[i,i]    = 1/s[i,i];
      invHHMdet = invHHMdet*s[i,i];
   endif;
   i = i+1;
endo;

invHHM  = u*s*u';
sigmult = u*sqrt(s);

"Determinant of minus Hessian";
invHHMdet;
"sqrt(Diagonal of Inverse Hessian)";
sqrt(diag(invHHM));

"Post Mode Penalty";
penalt = (rankHHM/2)*ln(2*pi) + 0.5*ln(invHHMdet);
penalt;

/* Initialize Output files
*/
omult = lpath $+ "\\" $+ lmodel $+ lprior $+ "mult";
create fhmult=^omult with MULT, npara, 8;
wr = writer(fhmult,sigmult);

closeall fhmult;
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/


proc (1) = fcn(para);
local lnpY, lnprio1, lnprio2, obsmean, obsvar;

/* likelihood 
*/
{lnpy, obsmean, obsvar} = evallbd( para,mspec,T0,YY);

/* Evaluate the Prior density
*/      
  lnprio1 = priodens( para, pmean, pstdd, pshape);
  lnprio2 = priomuphi( para );

retp(real(lnpY+lnprio1+lnprio2));  
endp;

/***************************************************************************
*/

proc (1) = priomuphi(para);
local muphi, lnprio;
muphi = para[7:8];
if mspec > 1;
   lnprio = -ln(2*pi) - 0.5*ln(det(muphi_v0))
            - 0.5*(muphi - muphi_m0)'*inv(muphi_v0)*(muphi - muphi_m0);
else;
   lnprio = 0;
endif;
retp(lnprio);
endp;



