/****************************************************/
/*                                                  */
/* Computing correlation betw measured prod and     */
/* output based on parameters of DSGE model         */
/*                                                  */
/*                                                  */
/****************************************************/

/* filename:    lbdpred2.g
** description: The program converts the DSGE parameters
**              into correlation betw measured prod and output
** created:     06/06/01
*/

new;
closeall;
library lbdlib, pgraph, user;
cls;

mspec  = 2;
mprior = 1;
mhrun  = "1";

/* Open files with Parameter Draws
*/
npara = 12;
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
opath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lmodel = "m" $+ ftos(mspec,"%*.*lf",1,0);
lprior = ftos(mprior,"%*.*lf",1,0);

lpara = lpath $+ lmodel $+ lprior $+ "par" $+ mhrun;
open fhpara = ^lpara for read;

/* Define some Constants
*/
nblock  = 100;         /* block size */
rowsel  = 1;


/* Initialize Output files
*/
otfp  = opath $+ lmodel $+ lprior $+ "tfp" $+ mhrun;

vnames = "CORR(dY,dTFP)" $|
         "CORR(dY,a*dX)" $|
         "CORR(dY,a*dA)" $|
         "COV(dY,a*dX)/COV(dY,dTFP)" $|
         "s(a*dX)/s(a*dA)";

create fhtfp = ^otfp with ^vnames, 5, 8;
 
/* Initialization
*/ 

eofloop = 0;
loopct  = 1;
totaldrawsused = 0;

do until eofloop; 

   /* Read blocks of size nblock
   */
   parasim = readr( fhpara, nblock );
   nread = rows(parasim);

   parasel = ( (seqa(1,1,nread) % rowsel) .NE 0);

   locate 1,1;
   "Block:        " loopct;
   "Size:         " nread;

   if minc(parasel) == 0;

     parasim = delif(parasim,parasel);
     nused   = rows(parasim);
     totaldrawsused = totaldrawsused + nused;

     "Elements Used:" nused;
     "";
     "Total Draws:  " totaldrawsused;

     CORsim   = zeros(nused,5);

     j = 1;
     do until j > nused;
      /* For each element of the block, compute the desired statistic
      */

      alp  = parasim[j,1];
      siga = parasim[j,npara-1];
      sigb = parasim[j,npara  ];

      /* Compute autocovariances and spectrum
      */
      {TTT, RRR, valid} = lbdsolv(parasim[j,.]', mspec);
      nstate = rows(TTT);
      /* State vector is of the form dy(t), dy(t-1), dl(t), dk(t+1),  dc(t), db(t)] [dx(t+1), dd(t)]
      ** Construct dY(t), alp*dA(t)+alp*dX(t), alp*dX(t), alp*dA(t)
      **           dY(t), dTFP(t), alp*dX(t), alp*dA(t)
      */
      /* Append state vector with X(t) and X(t-1)
      */
      TTT = (TTT ~ zeros(nstate,2) )|
            (zeros(1,6) ~ 1 ~ zeros(1,nstate-7+2) )|
            (zeros(1,nstate) ~ 1 ~ 0 );
      nstate = nstate + 2;

      RRR = RRR | zeros(2,cols(RRR));

      ZZ = zeros(4,nstate);
      ZZ[1,1] = 1; ZZ[1,2] = -1;
      ZZ[2,nstate-1] = alp; ZZ[2,nstate] = -alp;
      ZZ[3,nstate-1] = alp; ZZ[3,nstate] = -alp;

      HH = ( siga^2     ~ alp*siga^2   ~ 0  ~ alp*siga^2   )| 
           ( alp*siga^2 ~ alp^2*siga^2 ~ 0  ~ alp^2*siga^2 )| 
           ( 0          ~ 0            ~ 0  ~ 0            )|
           ( alp*siga^2 ~ alp^2*siga^2 ~ 0  ~ alp^2*siga^2 );

      QQ = ( siga^2 ~ 0 )|( 0 ~ sigb^2); 

      VV = ( siga^2     ~ alp*siga^2 ~ 0  ~ alp*siga^2 )|
           ( 0          ~ 0          ~ 0  ~ 0        );

      /* Initialize Autocovariances
      */
      GAMM0 = zeros(16,1);
      CORR0 = zeros(16,1);

      GA0  = inv(eye(nstate^2) - (TTT.*.TTT))*vec(RRR*QQ*RRR');
      GA0  = reshape(GA0,nstate,nstate);

      Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
      Gstar= sqrt(diag(Gl))*sqrt(diag(Gl)');

      GAMM0[.,1] = vecr(Gl);
      CORR0[.,1] = vecr(Gl./Gstar);
 
      CORsim[j,.] = CORR0[2]~CORR0[3]~CORR0[4]~Gl[3,3]/Gl[2,2]~sqrt(Gl[3,3]/Gl[4,4]);

      locate 10,1;
      $vnames';
      CORsim[j,.];

      j = j+1;
   endo;

   /* Write Output
   */
   wr = writer(fhtfp,CORsim);

  endif;

  loopct = loopct + 1;
  eofloop = eof(fhpara);
endo;


closeall fhtfp, fhpara;


