/***************************************************************/
/*                                                             */
/*     VAR analysis: sampling from the posterior,              */
/*                   Blanchard and Quah Decomposition          */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  minvargb.g
** created:   11/12/98
** modified:  08/24/99 adapted to LBD data
*/
new;
closeall;
library pgraph, user;
cls; 
_silent = 1;

lmodel= "m0";
lprior= "1";

/* Import data on output growth and inflation: series (nobs,2)
** observations from 1954:III to 1997:IV: 174 obs, 158 available for Lhours
*/
#include c:\projects\active\persist\gauss\prog_t03\loaddat2.g;

npak     = 10;              /* number of packages to save IRF  */
nsim     = 10000;           /* number of sim draws per package */
drawskip = 1;

nma   = 500;                /* number of MA coeff for B-Q Identification */
nirf  = 40;                 /* number of periods for IRF       */
ncovout = 8;                /* number of lags for acv output */


"Enter ID-Number for Simulation - Use 1 as Default";
locate 1,35;
mhrun = cons;
"To Run Gibbs Simulator Press 1";
keyin = keyw;

if keyin NE 49;
   goto plotirf;
endif;


/* Initialize Output files
*/
opath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";

oir11 = opath $+ lmodel $+ lprior $+ "ir11" $+ mhrun;
oir12 = opath $+ lmodel $+ lprior $+ "ir12" $+ mhrun;
oir21 = opath $+ lmodel $+ lprior $+ "ir21" $+ mhrun;
oir22 = opath $+ lmodel $+ lprior $+ "ir22" $+ mhrun;

ocory = opath $+ lmodel $+ lprior $+ "cory" $+ mhrun;
ocorh = opath $+ lmodel $+ lprior $+ "corh" $+ mhrun;
ogamm = opath $+ lmodel $+ lprior $+ "gamm" $+ mhrun;

create fhir11 = ^oir11 with IR11, nirf, 8;
create fhir12 = ^oir12 with IR12, nirf, 8;
create fhir21 = ^oir21 with IR21, nirf, 8;
create fhir22 = ^oir22 with IR22, nirf, 8;

create fhcory = ^ocory with CORY, ncovout, 8;
create fhcorh = ^ocorh with CORH, ncovout, 8; 
create fhgamm = ^ogamm with GAMM, 2, 8; 

cls;

/* Initialize Gibbs Sampler with OLS estimates
*/
CC_hat  = inv(XX'*XX)*XX'*YY;
Sig_hat = (YY-XX*CC_hat)'*(YY-XX*CC_hat)/nobs;
C_hat   = vec(CC_hat);
Sig_inv = inv(Sig_hat);
XXXX  = XX'*XX;

CC_sim = CC_hat;
C_sim  = C_hat;
Sigi_sim = Sig_inv;

/* Simulation loop
*/
totaldraws     = 0;
totaldrawsused = 0;

i = 1;
do until i > npak; 

  Asim11 = zeros(nsim,nirf); 
  Asim12 = zeros(nsim,nirf);
  Asim21 = zeros(nsim,nirf);
  Asim22 = zeros(nsim,nirf);

  CORYsim   = zeros(nsim,ncovout);
  CORHsim   = zeros(nsim,ncovout);
  GAMMsim   = zeros(nsim,2);

  j = 1;
  do until j > nsim;
   
   drawct = 1;
   do until drawct > drawskip;   

    val = 0;
    do until val == 1;

     /* draw psi
     */
     post = zeros(n_p1,1);
     expo = 0.5*(C_sim - C_bar)'*VCm_inv*(C_sim - C_bar);
     jj = 1;
     do until jj > n_p1;
        post[jj] = - 0.5*(nvar*k)*ln(p1_grid[jj]) - expo/p1_grid[jj];
        jj = jj+1;
     endo;
     post = post - ones(n_p1,1)*maxc(post);
     post = exp(post);
     post = post/sumc(post);

     z = rndu(1,1);
     jj = 1;
     do until jj > n_p1;
        if z le post[jj];
           p_sim = p1_grid[jj];
           jj = n_p1+1;
        else;
           jj = jj+1;
           post[jj] = post[jj]+post[jj-1];
        endif;
     endo;

     /* Draw C_sim
     */
     VC_inv  = VCm_inv/p_sim;
     VC_tili = VC_inv + Sigi_sim.*.XXXX;
     VC_til  = inv(VC_tili);
     C_til   = VC_til*(VC_inv*C_bar + (Sigi_sim.*.XXXX)*C_hat);
     CC_til  = reshape(C_til,nvar,k)';
     C_sim   = C_til + chol(VC_til)'*rndn(nvar*k,1);
     CC_sim  = reshape(C_sim,nvar,k)';
     CCt_sim = CC_sim';

     /* Draw sigi_sim
     */
     H = nobs*Sig_hat + (CC_sim - CC_hat)'*XXXX*(CC_sim - CC_hat);
     Z = chol(inv(H))'*rndn(nvar,nobs);
     Sigi_sim = Z*Z';
     Sig_sim  = inv(Sigi_sim);

     /* Check for stationarity
     */
     CCtilde =  CCt_sim[.,2+trd:k] |
                (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
     {laCCtil, vCCtil} = eigv(CCtilde);
     if maxc(abs(laCCtil)) < 0.99; 
        val = 1;
     endif;

    endo; 
    
    totaldraws = totaldraws+1;
    drawct = drawct + 1;
   endo;

   totaldrawsused = totaldrawsused + 1;

   /* Compute moving average coefficients
   */
   THE1 = eye(nvar);
   THE  = eye(nvar);
   sumTHE = THE*Sig_sim*THE';
   l = 1;
   do until l > nma;
      s = 1;
      THEl = zeros(nvar,nvar);
      do until s > minc( l|nlags );
         THEl = THEl + CCt_sim[.,2+trd+(s-1)*nvar:1+trd+s*nvar]*THE[.,1+(l-s)*nvar:2+(l-s)*nvar];
         s = s+1;
      endo;
      sumTHE = sumTHE + THEl*Sig_sim*THEl';
      THE = THE ~ THEl;
      THE1 = THE1 + THEl;
      l = l+1;
   endo;

   /* Calculate the coefficients of AA0-matrix
   */

   kap  = -THE1[1,2]/THE1[1,1];
   gam1 = SIG_sim[1,1] - kap*SIG_sim[1,2];
   gam2 = SIG_sim[1,2] - kap*SIG_sim[2,2];
   c2   = gam2^2/(gam1-gam2*kap);

   c    = sqrt(c2);

   if (gam2/c + kap*c) > 0;
      a = gam2/c + kap*c;
   else;
      c = -c;
      a = gam2/c + kap*c;
   endif;

   d = sqrt( (SIG_sim[1,2] - gam2 - kap*c2)/kap );
   b = kap*d;

   tst = SIG_sim[1,1] - a^2 - b^2 |
         SIG_sim[1,2] - a*c - b*d |
         SIG_sim[2,2] - c^2 - d^2 |
         THE1[1,1]*b + THE1[1,2]*d;

   AA0 = (a~b)|(c~d);

   /* Convert the THE coefficients into AA coefficients,
   ** and compute impulse response functions
   */
   sumAA2 = AA0*AA0';
   A11 = AA0[1,1];
   A12 = AA0[1,2];
   A11c = AA0[1,1];  /* cumulated IRF for output */
   A12c = AA0[1,2];  /* cumulated IRF for output */
   A21 = AA0[2,1];
   A22 = AA0[2,2];
   AAlc = AA0;
   l = 2;
   do until l > nirf;
      AAl = THE[.,(l-1)*2+1:l*2]*AA0;
      AAlc = AAlc + AAl;
      sumAA2 = sumAA2 + AAl*AAl';
      A11 = A11 ~ AAl[1,1];
      A12 = A12 ~ AAl[1,2];
      A11c = A11c ~ AAlc[1,1];
      A12c = A12c ~ AAlc[1,2];
      A21 = A21 ~ AAl[2,1];
      A22 = A22 ~ AAl[2,2];
      l = l+1;
   endo;

   /* Screen output
   */
   locate 1,1;
   "Block           " i "out of" npak;
   "Simulation step " j "out of" nsim;
   "";
   "Total Draws     " totaldraws;
   "Total Draws Used" totaldrawsused;

   if _silent == 0;
      "Tightness"       p_sim;
      "Coefficients";   CC_sim';
   endif;

   /* Save impulse response functions
   */
   Asim11[j,.] = A11c;   /* use cumulated IRF for output */
   Asim12[j,.] = A12c;   /* use cumulated IRF for output */
   Asim21[j,.] = A21;
   Asim22[j,.] = A22;

   /* Compute Autocovariance/-autocorrelation function
   */
   CCtilde =  CCt_sim[.,2+trd:k] |
              (eye(2*(nlags-1)) ~ zeros(2*(nlags-1),2));
   Sigtilde = (Sig_sim ~ zeros(nvar,nvar*(nlags-1)) )| zeros(nvar*(nlags-1),nvar*nlags);
   GA0tilde = inv(eye(4*nlags^2) - CCtilde.*.CCtilde)*vec(Sigtilde);
   GA0tilde = reshape(GA0tilde,nvar*nlags,nvar*nlags);
   CCtildel = CCtilde;
   
   GAMM0   = zeros(4,ncovout+1);
   CORR0   = zeros(4,ncovout+1);

   GA0     = GA0tilde[1:2,1:2];
   GA0star = (GA0[1,1] ~ sqrt(GA0[1,1]*GA0[2,2]) )|
             (sqrt(GA0[1,1]*GA0[2,2]) ~ GA0[2,2] );
   
   GAMM0[.,1] = vecr(GA0);
   CORR0[.,1] = vecr(GA0./GA0star);

   /* Compare Gamma(0) matrix of VAR and MA representation
   */
   if _silent == 0;
     {laCCtil, vCCtil} = eigv(CCtilde);
     locate 16,1; "Largest eigenvalue" maxc(abs(laCCtil));   
     locate 17,1; "Approximation Error Cov [Percent]"; 100*(sumAA2-GA0)./GA0;
   endif; 

   l = 1;
   do until l > ncovout;
      GAltilde = CCtildel*GA0tilde;
      GAl      = GAltilde[1:2,1:2];
      GAMM0[.,l+1] = vecr(GAl);
      CORR0[.,l+1] = vecr(GAl./GA0star);
      CCtildel = CCtildel*CCtilde;
      l = l+1;                          
   endo;

   CORYsim[j,.] = CORR0[1,2:1+ncovout];
   CORHsim[j,.] = CORR0[4,2:1+ncovout];
   GAMMsim[j,.] = GAMM0[1,1] ~ GAMM0[4,1];

   j = j+1;
  endo;
  
  /* Save the results
  */
  wr = writer(fhir11,Asim11);
  wr = writer(fhir12,Asim12);
  wr = writer(fhir21,Asim21);
  wr = writer(fhir22,Asim22);
  wr = writer(fhcory,CORYsim);
  wr = writer(fhcorh,CORHsim);
  wr = writer(fhgamm,GAMMsim);

  i = i+1;
endo;

closeall fhir11, fhir12, fhir21, fhir22,
         fhcory, fhcorh, fhgamm, fhpara;

"";
"Simulation complete - press key to continue";
keypress = keyw;


/* Plot the Impulse response functions
**************************************
*/
plotirf:
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";

nread = 10;
nburn = 11;
ti    = seqa(1,1,nirf);

ir_name  = "ir12" $|
           "ir22" $|
           "ir11" $|
           "ir21" ;

ir_label = "Output / Trans Shock" $|
           "Hours / Trans Shock"  $|
           "Output / Perm Shock"  $|
           "Hours / Perm Shock"   ;

graphset;
begwind;
margin(0.5,0.6,0.5,0.6);
window(2,2,0);
fonts("microb");
_ptitlht = 0.3;
_paxht = 0.25;
_pnumht = 0.2;
_protate = 0;
_pcolor =2 ;
_pltype = {6 1 2};

_plctrl = 4;
_plwidth = 1.8;

i = 1;
do until i > 4;

   lir = lpath $+ lmodel $+ lprior $+ ir_name[i] $+ mhrun;
   open fhir = ^lir for read;

   drawrow   = seekr(fhir,nburn);
   varirfsum = readr(fhir,1);

   ndraws  = 1;
   eofloop = 0;

   do until eofloop; 

     varirf    = readr(fhir,nread);
     varirfsum = varirfsum + sumc(varirf)';
     ndraws    = ndraws + rows(varirf);

     eofloop = eof(fhir);

   endo;
   closeall fhir;

   title(ir_label[i]);
   xy( ti,100*varirfsum'/ndraws );
   nextwind;

   i = i+1;

endo;
clear varirf;    

endwind;
