/******************************************************************/
/*                                                                */
/*                    Analyzing the Prior for MU and PHI          */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: muphprio.g
** created:  06/11/01
*/

new;
closeall;
cls;

muphi_scale = 25;
nsim = 500000;
hpd  = 0.9;

/* Load Prior for mu and phi
*/
ppath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";

psummary = ppath $+ "mu_phiq1" $+"s";
open fhpsummary = ^psummary for read;
muphi_m0 = readr(fhpsummary,1)';
muphi_s0 = readr(fhpsummary,1)';
muphi_v0 = readr(fhpsummary,2);

muphi_mult = chol(muphi_v0);

/* Generate data
*/
muphi_sim = (ones(nsim,1).*.muphi_m0') + rndn(nsim,2)*muphi_mult*muphi_scale;
muphi_hpd = hpdint(muphi_sim,hpd);
muphi_del = (muphi_sim[.,1] .< 0) .OR (muphi_sim[.,2] .< 0) .OR (muphi_sim[.,2] .GE 1);
muphi_sim2 = delif(muphi_sim,muphi_del);
muphi_hpd2 = hpdint(muphi_sim2,hpd);
ratio      = rows(muphi_sim2)/rows(muphi_sim);

/* Summary
*/
"Population Mean:   " muphi_m0';
"Population S.E.:   " muphi_s0';
"Scale:             " muphi_scale;      
"";
"Sim 1 - Mean:      " meanc(muphi_sim)';
"Sim 1 - S.E.:      " stdc(muphi_sim)';
"Sim 1 - HPD.:      "; muphi_hpd';
"";
"Sim 2 - Mean:      " meanc(muphi_sim2)';
"Sim 2 - S.E.:      " stdc(muphi_sim2)';
"Sim 2 - HPD.:      "; muphi_hpd2';
"";
"N(trunc) / N(full):" ratio;
clear muphi_sim, muphi_sim2, muphi_del;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort draws for column i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;

