/* filename:    tfpplot.g
** description: Compute posterior mean 
**              and plot filtered shock processes
*/

new;
closeall;
library user, lbdlib, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Simulation Draws
*/

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lmodel = "m2";
lprior = "1";
mhrun  = "1";

nblock  = 5;

oat  = lpath $+ lmodel $+ lprior $+ "at" $+ mhrun;
oxt  = lpath $+ lmodel $+ lprior $+ "xt" $+ mhrun;

open fhat = ^oat for read;
open fhxt = ^oxt for read;

drawrow = readr( fhat, 1); 
drawdim = cols(drawrow);
drawrow = seekr( fhat, 1);

/* Part 1: Compute the mean of x(i)
*/
atmean   = zeros(1,drawdim);
xtmean   = zeros(1,drawdim);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

   atblock = readr( fhat, nblock );
   xtblock = readr( fhxt, nblock );

   atmean  = atmean + sumc(atblock)';
   xtmean  = xtmean + sumc(xtblock)';

   ndraws = ndraws + rows(atblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhat);

endo;

atmean   = atmean'/ndraws;
xtmean   = xtmean'/ndraws;

/*atmean   = atmean - meanc(atmean);*/
tfpmean  = xtmean+atmean;

nobs     = rows(atmean);
ti       = seqa(1960,0.25,nobs);

/* Plot Impulse Response Functions
*/
graphset;
begwind;
margin(0,0,0.2,0.2);
window(2,1,0);
fonts("microb");

   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor =2 ;
   _pltype = {6 2 2 2};
   _protate = 1;
   _plctrl = 0;
   _plwidth = 4;
   _psymsiz = 5;

   _plegstr = "Measured TFP Process\000Technology Process";
   _plegctl = {1 4 1985 0.05};

   xlabel("Year");
   xtics(1960,1998,5,1);

   ylow  = -0.1;
   yhigh =  0.1;
   ytics(ylow,yhigh,0.02,1); 
   _pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960   ~ 0    ~ 2000   ~ 0     ~ 4 ~ 1 );

       
   title("Productivity");
   xy(ti,tfpmean-0.05~atmean-0.05);
   endwind;





